import 'package:flutter/material.dart';
import 'package:osam_flutter/osam_flutter.dart';

class LoadingWrapper extends StatelessWidget {
  final Widget child;
  final ValueNotifier<bool> isLoading;
  final Widget loadingWidget;

  const LoadingWrapper(
      {Key key, this.isLoading, this.child, this.loadingWidget})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        child,
        ValueListenableBuilder<bool>(
            valueListenable: isLoading,
            builder: (_, value, __) {
              return value ? loadingWidget : JustFadeOut(child: loadingWidget);
            })
      ],
    );
  }
}

class JustFadeOut extends StatefulWidget {
  final Widget child;

  const JustFadeOut({Key key, this.child}) : super(key: key);

  @override
  _JustFadeOutState createState() => _JustFadeOutState();
}

class _JustFadeOutState extends State<JustFadeOut> with AfterLayoutMixin {
  var animated = false;
  var blocked = true;

  @override
  Widget build(BuildContext context) {
    return blocked
        ? AnimatedOpacity(
            opacity: animated ? 0.0 : 1.0,
            duration: const Duration(milliseconds: 300),
            child: widget.child,
            onEnd: () {
              setState(() {
                blocked = false;
              });
            },
          )
        : const SizedBox.shrink();
  }

  @override
  void afterFirstLayout(BuildContext context) {
    setState(() {
      animated = true;
    });
  }
}
